﻿using System;
using System.Linq;
using System.Web.Security;
using Persistence;

namespace Service.Provider {
  class HeuristicLabMembershipProvider : MembershipProvider {
    public override string ApplicationName {
      get {
        throw new NotImplementedException();
      }
      set {
        throw new NotImplementedException();
      }
    }

    public override bool ChangePassword(string username, string oldPassword, string newPassword) {
      using (DataClassesDataContext db = DatabaseUtil.createDataClassesDataContext()) {
        // check database connection
        if (db == null) {
          return false;
        }
        try {
          // try to get user
          HeuristicLabUser u = db.HeuristicLabUsers.Single(x => x.UserName == username);
          if (u.ChangePassword(oldPassword, newPassword)) {
            // save user to database only if needed
            db.SubmitChanges();
            return true;
          } else {
            return false;
          }
        }
        catch (Exception) {
          return false;
        }
      }
    }

    public override bool ChangePasswordQuestionAndAnswer(string username, string password, string newPasswordQuestion, string newPasswordAnswer) {
      using (DataClassesDataContext db = DatabaseUtil.createDataClassesDataContext()) {
        // check database connection
        if (db == null) {
          return false;
        }
        try {
          // try to get user
          HeuristicLabUser u = db.HeuristicLabUsers.Single(x => x.UserName == username);
          if (u.ChangePasswordQuestionAndAnswer(password, newPasswordQuestion, newPasswordAnswer)) {
            // save user to database only if needed
            db.SubmitChanges();
            return true;
          } else {
            return false;
          }
        }
        catch (Exception) {
          return false;
        }
      }
    }

    public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status) {
      using (DataClassesDataContext db = DatabaseUtil.createDataClassesDataContext()) {
        // check database connection
        if (db == null) {
          status = MembershipCreateStatus.ProviderError;
          return null;
        }
        try {
          // check for duplicate entries
          if (db.HeuristicLabUsers.Count(x => x.UserName == username) > 0) {
            status = MembershipCreateStatus.DuplicateUserName;
            return null;
          }
          if (db.HeuristicLabUsers.Count(x => x.Email == email) > 0) {
            status = MembershipCreateStatus.DuplicateEmail;
            return null;
          }

          // create new user
          HeuristicLabUser u = new HeuristicLabUser(username, email, passwordQuestion, "");
          u.ChangePassword("INIT", password);
          u.ChangePasswordQuestionAndAnswer(password, passwordQuestion, passwordAnswer);
          // save user into database
          db.HeuristicLabUsers.InsertOnSubmit(u);
          db.SubmitChanges();

          // success
          status = MembershipCreateStatus.Success;
          return u;
        }
        catch (Exception) {
          // error
          status = MembershipCreateStatus.ProviderError;
          return null;
        }
      }
    }

    public override bool DeleteUser(string username, bool deleteAllRelatedData) {
      using (DataClassesDataContext db = DatabaseUtil.createDataClassesDataContext()) {
        // check database connection
        if (db == null) {
          return false;
        }
        try {
          // try to get user
          HeuristicLabUser u = 
            db.HeuristicLabUsers.Single<HeuristicLabUser>(x => x.UserName == username);

          // optionally delete related data
          if (deleteAllRelatedData) {
            db.HeuristicLabUserRole.DeleteAllOnSubmit<HeuristicLabUserRole>(u.HeuristicLabUserRole);
          }
          
          // delete user
          db.HeuristicLabUsers.DeleteOnSubmit(u);
          db.SubmitChanges();
          return true;
        }
        catch (Exception) {
          return false;
        }
      }
    }

    public override bool EnablePasswordReset {
      get { throw new NotImplementedException(); }
    }

    public override bool EnablePasswordRetrieval {
      get { throw new NotImplementedException(); }
    }

    public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords) {
      throw new NotImplementedException();
    }

    public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords) {
      throw new NotImplementedException();
    }

    public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords) {
      throw new NotImplementedException();
    }

    public override int GetNumberOfUsersOnline() {
      throw new NotImplementedException();
    }

    public override string GetPassword(string username, string answer) {

      throw new NotImplementedException();
    }

    public override MembershipUser GetUser(string username, bool userIsOnline) {
      throw new NotImplementedException();
    }

    public override MembershipUser GetUser(object providerUserKey, bool userIsOnline) {
      throw new NotImplementedException();
    }

    public override string GetUserNameByEmail(string email) {
      throw new NotImplementedException();
    }

    public override int MaxInvalidPasswordAttempts {
      get { throw new NotImplementedException(); }
    }

    public override int MinRequiredNonAlphanumericCharacters {
      get { throw new NotImplementedException(); }
    }

    public override int MinRequiredPasswordLength {
      get { throw new NotImplementedException(); }
    }

    public override int PasswordAttemptWindow {
      get { throw new NotImplementedException(); }
    }

    public override MembershipPasswordFormat PasswordFormat {
      get { throw new NotImplementedException(); }
    }

    public override string PasswordStrengthRegularExpression {
      get { throw new NotImplementedException(); }
    }

    public override bool RequiresQuestionAndAnswer {
      get { throw new NotImplementedException(); }
    }

    public override bool RequiresUniqueEmail {
      get { throw new NotImplementedException(); }
    }

    public override string ResetPassword(string username, string answer) {
      throw new NotImplementedException();
    }

    public override bool UnlockUser(string userName) {
      throw new NotImplementedException();
    }

    public override void UpdateUser(MembershipUser user) {
      throw new NotImplementedException();
    }

    public override bool ValidateUser(string username, string password) {
      throw new NotImplementedException();
    }
  }
}
